"""Main fastapi module."""

from collections.abc import AsyncGenerator
from contextlib import asynccontextmanager
from typing import Literal

from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware

from .auth import router as auth_router
from .constants import (
    LOGGER_SERVICE,
    LOGGER_SERVICE_HANDLE,
    LogEntryType,
    LoggerResult,
    LogReferenceType,
)
from .contact import router as contact_router
from .oidc import router as oidc_router
from .private import router as private_router
from .settings import get_cache, get_logger_client, get_settings
from .verification import router as verification_router


@asynccontextmanager
async def lifespan(app: FastAPI) -> AsyncGenerator[None, None]:
    """Provide lifespan for the FastAPI app."""
    # Call get_settings to ensure all the required settings are provided
    get_settings()

    # Register service and log entries to logger
    logger_client = get_logger_client(get_settings())
    logger_client.register_service(LOGGER_SERVICE, handle=LOGGER_SERVICE_HANDLE)
    logger_client.register_log_entry_types(LOGGER_SERVICE, log_entry_types=LogEntryType)
    logger_client.register_results(LOGGER_SERVICE, results=LoggerResult)
    logger_client.register_object_reference_types(reference_types=LogReferenceType)

    # Check redis cache
    await get_cache(get_settings()).ping()

    yield


app = FastAPI(lifespan=lifespan)
app.add_middleware(
    CORSMiddleware,
    allow_origins=get_settings().allowed_origins,
    allow_methods=["*"],
    allow_headers=["*"],
)
app.include_router(auth_router)
app.include_router(contact_router)
app.include_router(oidc_router)
app.include_router(verification_router)
app.include_router(private_router)


@app.get("/healthcheck/")
async def healthcheck() -> Literal[True]:
    """Simply return true.

    You can call this endpoint to make sure that API is up and running.
    """
    return True
